%MACRO BREAKDN(CLASS=,VAR=,OUT=OUT,                                             
               DATA=_LAST_,PRINT=NO,SFIRST=VAR,where=,nzero=);                         
                                                                                
/*                                                                              

   Jan 2013 - Check for duplicate variables in var list

   Feb 05  - added 1st and 3rd quartiles (LQ and UQ)
   15OCT01 - added nzero option, if set = y then missing n's
             get the value 0 rather than .
   Changes made to add median for Version 8 - will not work in Version 6

   CLASS = CLASS VARIABLES  ( EX. CLASS= SEX 2T RACE 4)                         
   VAR = ANALYSIS VARIABLES ( EX. VAR= CHOL DBP CIGS)                           
   OUT = OUTPUT DATA SET    ( EX. OUT = TABLE1)                                 
   SFIRST = ORDER VARIABLES ARE STORED (EX. SFIRST=VAR OR                       
            SFIRST = CLASS)                                                     
*/                                                                              
                                                                                

                                                                                
%LET CLASS = %UPCASE(&CLASS);                                                   
%LET nzero = %UPCASE(&nzero);                                                   
%LET SFIRST = %UPCASE(&SFIRST);                                                 
%let print = %upcase(&print);

DATA _NULL_ ;                                                                   

%finddups(&var);
%if &_dups = 1 %then %do ;
 ERROR '*** DUPLICATE VARIABLES IN VAR LIST ***';             
 ABORT;                                                                         
%end;

 
                                                                                
                                                                                
%LET XCLASS = %STR(); %LET NCAT=%STR() ; %LET TOT = %STR();                     
%DO I = 1 %TO 6 ;                                                               
 %LET TEMP = %SCAN(&CLASS,&I);                                                  
 %IF &TEMP = %STR() %THEN %GOTO ENDINIT;                                        
 %IF &I = 1 OR &I = 3 OR &I = 5 %THEN %LET XCLASS=&XCLASS &TEMP;                
 %IF &I = 2 OR &I = 4 OR &I = 6 %THEN %DO ;                                     
  %LET NCAT =&NCAT %SCAN(&TEMP,1,T);                                            
  %IF %INDEX(&TEMP,T) GT 0 %THEN %LET TOT = &TOT 1 ;                            
  %IF %INDEX(&TEMP,T) EQ 0 %THEN %LET TOT = &TOT 0 ;                            
 %END;                                                                          
%END;                                                                           
%ENDINIT:                                                                       
%LET CLASS = &XCLASS;                                                           
                                                                                
%DO I = 1 %TO 200;                                                              
 %LET TEMP = %SCAN(&VAR,&I);                                                    
 %IF &TEMP = %STR() %THEN %GOTO ENDNVAR;                                        
%END;                                                                           
%ENDNVAR:                                                                       
%LET NVAR = %EVAL(&I-1);                                                        
                                                                                
%DO I = 1 %TO 100;                                                              
 %LET TEMP = %SCAN(&NCAT,&I);                                                   
 %IF &TEMP = %STR() %THEN %GOTO ENDNCAT;                                        
%END;                                                                           
%ENDNCAT:                                                                       
%LET NUMCAT = %EVAL(&I-1);                                                      
                                                                                
%DO I = 1 %TO 100;                                                              
 %LET CLASS&I = %SCAN(&CLASS,&I);                                               
 %LET TOT&I = %SCAN(&TOT,&I) ;                                                  
 %IF &&CLASS&I = %STR() %THEN %GOTO ENDCL;                                      
%END;                                                                           
%ENDCL:                                                                         
%LET NCLASS = %EVAL(&I-1);                                                      
                                                                                
%IF &NUMCAT NE &NCLASS %THEN %DO;                                               
 ERROR '*** NUMBER OF CATEGORIES NE NUMBER OF CLASS VARIABLES ***';             
 ABORT;                                                                         
%END;                                                                           
                                                                                
PROC SUMMARY DATA=&DATA;                                                        
 CLASS &CLASS;                                                                  
 VAR &VAR;                                                                      
 OUTPUT OUT = &OUT N      = XN1-XN&NVAR                                         
                   MEAN   = XM1-XM&NVAR                                         
                   MEDIAN = XMED1-XMED&NVAR
		   Q1     = XLQ1-XLQ&NVAR
		   Q3     = XUQ1-XUQ&NVAR
                   STD    = XS1-XS&NVAR                                         
                   STDERR = XSE1-XSE&NVAR                                       
                   SUM    = XSUM1-XSUM&NVAR                                     
                   MAX    = XMAX1-XMAX&NVAR                                     
                   MIN    = XMIN1-XMIN&NVAR  ;                                  
 &where;                                                                        
%IF &PRINT = YES %THEN %DO;                                                     
 PROC PRINT;                                                                    
%END;                                                                           
                                                                                
DATA &OUT;                                                                      
                                                                                
%LET CNT = 1;                                                                   
%DO I = 1 %TO 100;                                                              
 %LET TEMP = %SCAN(&NCAT,&I);                                                   
 %IF &TEMP = %STR() %THEN %GOTO ENDCNT;                                         
 %LET TEMP = %EVAL(&TEMP+&&TOT&I);                                              
 %LET NCAT&I = &TEMP;                                                           
 NCAT&I = &TEMP;                                                                
 TOT&I = &&TOT&I;                                                               
 %LET CNT = %EVAL(&TEMP*&CNT);                                                  
%END;                                                                           
%ENDCNT:                                                                        
%LET NCAT = &CNT;                                                               
%LET DIM = %EVAL(&NVAR*&NCAT);                                                  
                                                                                
                                                                                
ARRAY NCAT(*) NCAT1-NCAT&NCLASS;                                                
ARRAY TOT(*) TOT1-TOT&NCLASS;                                                   
ARRAY CLASS(*) &CLASS;                                                          
ARRAY YN(*) N1-N&DIM;                                                           
ARRAY YM(*) M1-M&DIM;                                                           
ARRAY YMED(*) MED1-MED&DIM;
ARRAY YLQ(*) LQ1-LQ&DIM;
ARRAY YUQ(*) UQ1-UQ&DIM;
ARRAY YS(*) S1-S&DIM;                                                           
ARRAY YSE(*) SE1-SE&DIM;                                                        
ARRAY YSUM(*) SUM1-SUM&DIM;                                                     
ARRAY YMIN(*) MIN1-MIN&DIM;                                                     
ARRAY YMAX(*) MAX1-MAX&DIM;                                                     
                                                                                
ARRAY XN(*) XN1-XN&NVAR;                                                        
ARRAY XM(*) XM1-XM&NVAR;                                                        
ARRAY XMED(*) XMED1-XMED&NVAR;
ARRAY XLQ(*) XLQ1-XLQ&NVAR;
ARRAY XUQ(*) XUQ1-XUQ&NVAR;
ARRAY XS(*) XS1-XS&NVAR;                                                        
ARRAY XSE(*) XSE1-XSE&NVAR;                                                     
ARRAY XSUM(*) XSUM1-XSUM&NVAR;                                                  
ARRAY XMIN(*) XMIN1-XMIN&NVAR;                                                  
ARRAY XMAX(*) XMAX1-XMAX&NVAR;                                                  
                                                                                
DO I = 1 TO TOTOBS ;                                                            
 SET &OUT POINT = I NOBS=TOTOBS;                                                
 DO J = 1 TO DIM(CLASS);                                                        
  IF CLASS(J) = . AND TOT(J) NE 1 THEN GOTO SKIP ;                              
  IF CLASS(J) = . THEN CLASS(J) = NCAT(J);                                      
 END;                                                                           
 %IF &NCLASS = 1 %THEN %DO;                                                     
  START = &CLASS1;                                                              
 %IF &SFIRST=VAR %THEN %DO ;                                                    
  START = (START-1)*(&NVAR) + 1;                                                
 %END;                                                                          
 %END;                                                                          
 %IF &NCLASS = 2 %THEN %DO;                                                     
  START = &CLASS2  + (&CLASS1 -1) * &NCAT2 ;                                    
 %IF &SFIRST=VAR %THEN %DO;                                                     
  START = (START-1)*(&NVAR) + 1;                                                
 %END;                                                                          
 %END;                                                                          
 %IF &NCLASS = 3 %THEN %DO;                                                     
  START = &CLASS3  + (&CLASS2 -1) * &NCAT3 +                                    
          (&CLASS1 - 1) * &NCAT3*&NCAT2;                                        
 %IF &SFIRST = VAR %THEN %DO ;                                                  
  START = (START-1)*(&NVAR) + 1;                                                
 %END;                                                                          
 %END;                                                                          
                                                                                
 DO K = 1 TO &NVAR;                                                             
  %IF &SFIRST = VAR %THEN %DO;                                                  
  INDEX =  START + K - 1 ;                                                      
  %END;                                                                         
  %IF &SFIRST = CLASS %THEN %DO ;                                               
  INDEX =  &NCAT* (K-1)  + START ;                                              
  %END;                                                                         
  YN(INDEX) = XN(K);                                                            
  YM(INDEX) = XM(K);                                                            
  YMED(INDEX) = XMED(K);
  YLQ(INDEX) = XLQ(K);
  YUQ(INDEX) = XUQ(K);
  YS(INDEX) = XS(K);                                                            
  YSE(INDEX) = XSE(K);                                                          
  YSUM(INDEX) = XSUM(K);                                                        
  YMIN(INDEX) = XMIN(K);                                                        
  YMAX(INDEX) = XMAX(K);                                                        
 END;                                                                           
SKIP:                                                                           
END;                                                                            
KEEP N1-N&DIM M1-M&DIM MED1-MED&DIM LQ1-LQ&DIM UQ1-UQ&DIM S1-S&DIM SE1-SE&DIM
     SUM1-SUM&DIM MIN1-MIN&DIM MAX1-MAX&DIM;                                    
%if &nzero = Y %then %do;
 do z = 1 to &dim;
  if yn(z) = . then yn(z) = 0;
 end;
%end;
OUTPUT;                                                                         
STOP;                                                                           
                                                                                
%MEND;                                                                          
