%macro chisqp(data=_last_, dlist=, group=, clist=, where=, print=NOPRINT, 
              pdata=, test=);                                                  
                                                                                
/* This macro puts the p-values for chi-square tests into a                     
   SAS data set. The p-values are the Cochran-Mantel-Haenszel                   
   which adjusts for a set of other categorical variables.                      
                                                                                
   data   = data set to read                                                    
   dlist  = list of categorical dependent variables                             
   group  = categorical variable to be tested for relationship                  
            with each dependent variable                                        
   clist  = covariate list of categorical variables                             
   pdata  = output data set with p-values, 1 for each dep var.                  
            This dataset has 1 observation, with variables                      
            p1-p(N), N the number of dep var's.                                 
   test   = G (General Association), C (correlation 1df),                       
            R (Row Means Differ)                                                
   where   = where clause to subset observations                                
               (set where to "where sex = 1" to do analyses on men)
   print = to see the freq tables, you need to change the default setting
           from NOPRINT to Y or y
                                                                                
*/                                                                              
             
/* decide what kind of TEST you request */
%let test = %upcase(&test);                                                     
                                                                                
%if &test=%str() %then %let test=G;                                             
                                                                                
/* store the FREQ table in CHISQ */
filename chisq '_chisq_.lst';
                                                                                
%ntok(&clist,ncov,vname=clist);
%ntok(&dlist,ndep);

/* allow the NOPRINT in PROC FREQ as an option */
%if %upcase(&print) ne NOPRINT %then %let print = %str();

/* redirect the output to CHISQ */
proc printto print=chisq new;                                                   
proc freq data=&data ;                                                          
 tables                                                                         
 %do i = 1 %to &ncov;                                                           
 &&clist&i *                                                                    
 %end;                                                                          
  (&group)*(&dlist)/cmh &print ;                                                
  &where ;

/* set the output back to default */
proc printto ;                                                                  
data &pdata ;                                                                   
 infile chisq missover ;                                                        
 keep  p1-p&ndep numobs ;                                                                       
 test = "&test";                                                                
 length record6 $32.;
 input record1 $                                                                
       record2 $                                                                
       record3 $                                                                
       record4 $                                                                
       record5 $                                                                
       record6 $                                                                
       record7 $                                                                
       record8 $                                                                
       ;                            
 record1= upcase(record1);
 record2= upcase(record2);                                                      
 record3= upcase(record3);                                                      

 array p(*) p1-p&ndep;
 retain namevar p1-p&ndep;

 /* check to see whether there is any missing SUMMARY table for dep var list*/
 if record1 = 'SUMMARY' then namevar=upcase(record6);
 %do j = 1 %to &ndep;
   %let v = %scan(&dlist, &j);
   %let v = %upcase(&v);
   if namevar ="&v" then do;
      if test = 'G' and record2 = 'GENERAL' then do ;                           
         p(&j) = 1- probchi(input(record5,8.0),input(record4,4.0)) ;
	 numobs+1;
         output;
      end;                                                                     
      if test='C' and record2 = 'NONZERO' then do ;                             
         p(&j) = 1- probchi(input(record5,8.0),input(record4,4.0)) ;
	 numobs+1;
         output; 
      end;                                                                      
      if test='R' and record2 = 'ROW' and record3 = 'MEAN' then do;         
         p(&j) = 1- probchi(input(record7,8.0),input(record6,4.0)) ;
	 numobs+1;
         output;
      end;                                                                      
   end;
%end;
run;                                                                            

/* get the last line in the dataset*/
data &pdata;
     set &pdata end=last;
     by numobs;
     keep p1-p&ndep ;
     if last;

*proc print data=&pdata;
%mend;                                                                          
