%macro cmhp(data=_last_, dlist=, group=, clist=, where=, print=NOPRINT, 
              pdata=, test=);                                                  
                                                                                
/* 


  This macro puts the p-values for chi-square tests into a                     
   SAS data set. The p-values are the Cochran-Mantel-Haenszel                   
   which adjusts for a set of other categorical variables.                      
   This macro can be used rather than chisqp. Written using
   ods rather than proc printto.
                                                                                
   data   = data set to read                                                    
   dlist  = list of categorical dependent variables                             
   group  = categorical variable to be tested for relationship                  
            with each dependent variable                                        
   clist  = covariate list of categorical variables                             
   pdata  = output data set with p-values, 1 for each dep var.                  
            This dataset has 1 observation, with variables                      
            p1-p(N), N the number of dep var's.                                 
	    x1-x(N), Chi-square values
	    df1-df(N) Degrees of freedom
   test   = G (General Association), C (correlation 1df),                       
            R (Row Means Differ)                                                
   where   = where clause to subset observations                                
               (set where to "where sex = 1" to do analyses on men)
   print = to see the freq tables, you need to change the default setting
           from NOPRINT to Y or y
                                                                                
*/                                                                              
             
/* decide what kind of TEST you request */
%let test = %upcase(&test);                                                     
                                                                                
%if &test=%str() %then %let test=G;                                             
                                                                                
                                                                                
%ntok(&clist,ncov,vname=clist);
%ntok(&dlist,ndep,vname=dlist);

/* allow the NOPRINT in PROC FREQ as an option */
%if %upcase(&print) ne NOPRINT %then %let print = %str();

ods output cmh = _cmh_ ;
proc freq data=&data ;                                                          
 tables                                                                         
 %do i = 1 %to &ncov;                                                           
 &&clist&i *                                                                    
 %end;                                                                          
  (&group)*(&dlist)/cmh &print ;                                                
  &where ;
run;
ods output close;

* Removes rows other than type of test requested;
data _cmh_;
 set _cmh_;

 %if &test = G %then %do ;
  where AltHypothesis = 'General Association';
 %end;
 %else
 %if &test = C %then %do;
  where AltHypothesis = 'Nonzero Correlation';
 %end;
 %else
 %if &test = R %then %do;
  where AltHypothesis = 'Row Mean Scores Differ';
 %end;
run;

* Puts data in 1-record dataset. If test not done, e.g. no events
  in any group than statistics will be set to missing;

data &pdata; 
 retain p1-p&ndep x1-x&ndep df1-df&ndep;
 array p{*} p1-p&ndep;
 array x{*} x1-x&ndep;
 array xdf{*} df1-df&ndep;

 set _cmh_ end=eof;
 %do i = 1 %to &ndep;
  if scan(lowcase(table),-1) = lowcase("&&dlist&i") then do;
  index = &i;
 end;
 %end;
  p{index} = Prob;
  x{index} = Value;
  xdf{index} = DF;

  keep p1-p&ndep x1-x&ndep df1-df&ndep;

  if eof then output; 
run;

proc delete data=_cmh_;
run;

%mend;



