
/******************************************************************************

 Sept 2016 (GG) Changes to set p-values to missing if no events in any var
 August 2016 (GG) Changed code to make it simpler
 March 2015 (GG);
 Macro to produce p-values from Fishers Exact test. User gives list of
 dependent variables and independent variable (group).

 Parameters:

 data = dataset. Must be specified
 dlist = list of dependent categorical variables, usually 1/2 or 0/1)
 group = independent group variables
 where = complete where clause including where
 out = dataset with p-values (p1-p&ndep)

 Note: If there are no events for a dependent variable then the pvalue is
       set to missing (no test is actually done). If there are events in 
       only one group the test is done and a p-value is generated.

******************************************************************************/

%macro fisherp(data=, dlist=, group=, where=,  out=);                                                 

* Find number of dependent variables;
%ntok(&dlist,ndep,vname=dlist);

proc means data=&data noprint;
 var &dlist;
 where &group ne .;
 output out=check sum=sum_events1 - sum_events&ndep;
run;
data check;
 set check;
 tevents = sum(of sum_events1-sum_events&ndep);
 keep tevents;
 call symputx('tevents',put(tevents,5.0));
run;

%if %eval(&tevents) > 0 %then %do ;

ods output fishersexact = _test_; 
proc freq data=&data;
 tables &group*(&dlist);
 exact fisher;
 &where ;
run;
ods output close;

data _test_;
 set _test_;
 where Name1 = 'XP2_FISH';
run;

data &out;
 retain p1-p&ndep;
 array p{*} p1-p&ndep;

 set _test_ end=eof;
 %do i = 1 %to &ndep;
  if scan(lowcase(table),-1) = lowcase("&&dlist&i") then do;
  index = &i;
  end;
 %end;
 p{index} = nValue1;
 keep p1-p&ndep;

 if eof then output;
run;

proc delete data=_test_;
run;
%end;
%else %do; 
data &out;
 array p{*} p1-p&ndep;
 put "**** No events in any group for all dependent variables";
 put "**** All p-values set to missing";
run;
%end;

%mend fisherp;                                                                          
