/**************************************************************************
   Updated March 2001 to add ability to output statistics from estimate and 
   contrast statements. Macro re-written to use the output delivery system
   (ODS).  Works in Version 8 only.
**************************************************************************/

%macro glmp(data=_last_,
            class=,
	    dlist=, ilist=, blist=,
            factor=,pdata=&factor,
	    lsmean=,lsclass=&class,lsdata=,
            estimate=,estdata=,
            contrast=,condata=,
	    where = );

/* Overview:	    
 
   GLMP runs PROC GLM(s) and stores statistics generated from the
   analysis into one or more SAS datasets. Statistics include
   p-values from factors in the model, least-square (adjusted means)
   and contrast estimates and significance levels.

   Description of Parameters:
 
     data   = SAS data set to be read
     class  = class variables for GLM
     dlist  = dependent variable list
     ilist  = independent variable list. This list will be used for each
              dependent variable given in dlist.
     blist  = (optional) if given, the i'th variable is added as
              a covariate to the i'th analysis. Used mostly to
              adjust for the baseline value of the dependent
              variable.
     factor = independent variable(s) for which statistics are to be
              stored.
     pdata  = SAS dataset(s) in which statistics are stored for the variables
              listed in factor.  There is one dataset for each factor with one
              observation in each dataset. The variables on this datset are:

	      P1-Pn = The p-values for the i'th analyses (n analyses)
	      F1-Fn = The F-values for the i'th analyses
	      
     lsmean = set to YES if least-square means are desired, 
     lsclass= the class variable for which LS means are desired. 
     lsdata = SAS dataset containing the LS means. This contains
              only one observation containing the variables:

	      LSM1-LSMn = The LS means
	      LSMSE1-LSMSEn = The LS mean standard errors
	      n = the number of levels of the class variable times 
	          the number of dependent variables 
              For example, if your class level has 5 levels
              and 2 dependent variables are analysed then n=10; LS1-LS5
              contain the least square means for the first dependent variable
              and LS6-LS10 contain the LS means for the second dep. variable.
 
     where   = where clause to subset observations
               (set where to "where sex = 1" to do analyses on men)

             The following are new (March 2001):

     estimate= This uses the estimate statement in GLM to perform 1 df contrasts
               Multiple contrasts are separated by a /.
	       Ex: estimate = '1 v 3' trt 1 0 -1/ '2 v 3' trt 0 1 -1
	       This would perform two contrasts, the first comparing the first
	       and third levels of trt, and the second comparing the second
	       and third levels of trt. 

     estdata = This is the dataset the estimates (contrasts) are stored.
               The variables on this datset are:

	       E1-E? = contrast values (estimates)
	       S1-S? = standard error of the estimates
	       T1-T? = t-statistic
	       P1-P? = p-values
     ? is the total number of estimate statements times the number of
     dependent variables. They are stored in the order generated. If 
     there are 2 dependent variables and 2 contrasts then
     
     e1 = estimate for analyses 1, contrast 1
     e2 = estimate for analyses 1, contrast 2
     e3 = estimate for analyses 2, contrast 1
     e4 = estimate for analyses 2, contrast 2

     contrast= This uses the contrast statement in GLM to perform 
               1 or >1 df contrasts.  Multiple contrasts are separated by a /.
	       Ex: >1 df contrasts are given like this:
	       contrast=%str('2 df test' trt 1 0 -1, trt 1 -1 0)
	       1 df contrasts do not %str around the text

     condata = This is the dataset the contrasts are stored.
               The variables on this datset are:

	       F1-F? = F-statistics
	       P1-P? = p-values
	       DF1-DF? = Degree of freedom of constrasts

     ? is the total number of contrast statments times the number of
     dependent variables. They are stored in the same order as in the
     estimate statements (see above).
	       
	       
     Note:    If blist is not given then only one GLM is run with
              multiple dependent variables. If blist is given then
              a seperate GLM is run for each dependent variable.
 
     Examples:
 
     %let list = qlghid qlefd qlmhid qlgfid ;
     %let listb= qlghi00 qlef00 qlmhi00 qlgfi00;
 
     %glmp(data=temp,class=group clinic ,dlist=&list,
           ilist=group clinic, factor=group,pdata=qlunad);
 
     %glmp(data=temp,class=group clinic ,dlist=&list,
           ilist=group clinic,blist=&listb,
           factor=group,pdata=qladj);
 
     %glmp(data=temp,class=group ,dlist=&list,
           ilist=group ,blist=&listb,
           factor=group,pdata=qladj,lsmean=yes,lsdata=lsmeans);

     %glmp (data=temp, class=trt clinic, 
            dlist = chold ldld hdld trigd,
            ilist = trt clinic,
            blist = chol2s ldl2s hdl2s trig2s,
            factor = trt, pdata=trt,
            estdata = est,  
	    estimate = '1 v 3' trt 1 0 -1/ '2 v 3' trt 0 1 -1
            condata = con, 
	    contrast=%str('2 df test' trt 1 0 -1, trt 1 -1 0));

Here is a complete program with the output (output followed by program).

        Change in Blood Lipids for Three Active Drug Groups and Placebo





                     Mean Change in Lipids       P-Values vs Placebo    P-value
Change in Lipids   --------------------------    -------------------    Active
(mg/dl)             BB    Diur    ACE  Placebo    BB    Diur    ACE     (2 DF)
-----------------  -----  -----  -----  -----    -----  -----  -----    -------

Total Plasma       -11.7   -6.7   -8.0   -5.1    0.029  0.721  0.180     0.275

LDL Cholesterol    -10.6   -5.1   -5.9   -3.6    0.007  0.667  0.120     0.137

HDL Cholesterol      0.2    2.0    2.6    1.4    0.089  0.286  0.049     0.003

Triglycerides       -6.4  -18.4  -23.6  -14.5    0.020  0.485  0.452     0.008


        N           129    124    129    227
data temp;                                                                      
 set tomhsdat.rand;                                                             
 set tomhsdat.lipid;                                                           
 if group in(1,2,5,6);
 if group = 1 then cat = 1;
 if group = 2 then cat = 2;
 if group = 5 then cat = 3;
 if group = 6 then cat = 4;
run;                                                                            
proc means; class cat; var chold ldld hdld trigd;
%breakdn(data=temp, class=cat 4, var=chold ldld hdld trigd,
         sfirst=class, out=out1);
%glmp (data=temp, class=cat stratum clinic, 
       dlist = chold ldld hdld trigd,
       ilist = cat stratum clinic,
       blist = chol2s ldl2s hdl2s trig2s,
       factor = cat, pdata=cat,
       estdata = est,  
       estimate = '1 vs 4' cat 1 0 0 -1/ '2 vs 4' cat 0 1 0 -1/ 
                  '3 vs 4' cat 0 0 1 -1,
       condata = con, 
       contrast= %str('RX Dif 2df' cat 1 -1 0 0, cat 0 1 -1 0));
%report(repfile='rxlipids.table');
%colset(18 7 7 7 7 2x 7 7 7 2x 9);
%move('Change in Blood Lipids for Three Active Drug Groups and Placebo',
       col=1-0, line=3);
%move('BB':'Diur':'ACE':'Placebo', col=2.5, line=11, u=1);
%move('BB':'Diur':'ACE', col=6.8, line=11, u=1);
%move('Mean Change in Lipids', col=2-5, line=9, u=1);
%move('P-Values vs Placebo', col=6-8, line=9, u=1);
%move('P-value':'Active':'(2 DF)':'-------', col=9, line=9L4);
%move('Change in Lipids', col=1, line=10, center=2);
%move('(mg/dl)', line=11, center=2, u=1);
%move('Total Plasma ':
      'LDL Cholesterol ':
      'HDL Cholesterol ':
      'Triglycerides ', col=1, center=2, line=14L4, pat=1st/1);
%move('N', col=1, line=23);

set out1;
%nmove(n1-n4, col=2.5, line=23, fmt=4.0);
%nmove(m1-m16, col=2.5, line=14L4, pat=1st/1, mfirst=c);

set est;
%nmove(p1-p12, col=6.8, fmt=6.3,mfirst=c);
set con;
%nmove(p1-p4, col=9, fmt=6.3)

*/
 
ods output ModelAnova (match_all=_anova_ persist=proc)= _anova_ ;
ods output LSMeans (match_all=_lsmeans_ persist=proc)= _lsmeans_ ;
ods output Estimates (match_all=_estimates_ persist=proc)= _estimates_ ;
ods output Contrasts (match_all=_contrasts_ persist=proc)= _contrasts_ ;

data _null_;

%local _lsmeans_ _estimates_ _contrasts_ ;
%local i j k var;
%let dlist = %upcase(&dlist) ;
%let factor = %upcase(&factor) ;
%let lsclass = %upcase(&lsclass);
%let lsmean  = %upcase(&lsmean);
 
%ntok(&factor,nfactor,vname=factor);
%ntok(&pdata ,npdata ,vname=pdata);
%ntok(&dlist ,ndep);
%ntok(&blist ,nbas,vname=bl);
%ntok(&estimate ,nest,vname=estimate,token=%str(/));
%ntok(&contrast ,ncon,vname=contrast,token=%str(/));
 
%if &class eq %str() or &dlist eq %str() or %length(&ilist) = 0
    or %length(&factor)  eq 0  or &pdata eq %str() %then %do ;
 ERROR '*** SOME PARAMETERS NOT SPECIFIED - SEE MACRO CALL ***';
 ABORT;
 ENDSAS;
%end;
 
%if &blist ne %str() %then %do;
%if &ndep ne &nbas %then %do;
 ERROR '*** NUMBER OF DEPENDENT AND BASELINE VARIABLES NOT = ***';
 ABORT;
 ENDSAS;
%end;
%end;
 
%if &npdata ne &nfactor %then %do ;
 ERROR '*** NUMBERS OF FACTORS AND OUTPUT DATASETS NOT =  ***';
 ABORT;
 ENDSAS;
%end;
 
 
%if &blist = %str() %then %do;
 proc glm data=&data ;
  class &class;
  model &dlist = &ilist /ss3 solution ;

 %if &lsmean = YES %then %do;
  means &lsclass/deponly;
  lsmeans &lsclass/ stderr;
 %end;

 %if %eval(&nest) gt 0 %then %do;
 %do i = 1 %to &nest; 
  estimate &&estimate&i ;
 %end;
 %end;

 %if %eval(&ncon) gt 0 %then %do;
 %do j = 1 %to &ncon;
  contrast &&contrast&j;
 %end;
 %end;

 %if %length(&where) ne 0 %then %do;
 &where;
 %end;
%end;
 
%if &blist ne %str() %then %do ;
 %do i = 1 %to &ndep;
  %let dep = %scan(&dlist,&i);
  proc glm data=&data ;
   class &class;
   model &dep = &ilist &&bl&i/ss3 solution;

  %if &lsmean = YES %then %do;
   means &lsclass/deponly;
   lsmeans &lsclass/ stderr;
  %end;
  
 %if %eval(&nest) gt 0 %then %do;
 %do j = 1 %to &nest; 
  estimate &&estimate&j ;
 %end;
 %end;

 %if %eval(&ncon) gt 0 %then %do;
 %do j = 1 %to &ncon;
  contrast &&contrast&j; 
 %end;
 %end;

 %if %length(&where) ne 0 %then %do;
  &where;
 %end;

 %end;
%end;
 
data _anova_;
 length Dependent $32.;
 set &_anova_;
run;
*proc contents;
 
%do j = 1 %to &nfactor ;
 data &&pdata&j;
  set _anova_;
  if upcase(Source) = "&&factor&j" ;

 data &&pdata&j ;
  keep source p1-p&ndep f1-f&ndep;
  array p(*) p1-p&ndep;
  array af(*) f1-f&ndep;
  dO i = 1 to totobs;
   set &&pdata&j point=i nobs=totobs;
   %do i = 1 %to &ndep ;
    %let var = %scan(&dlist,&i);
    if upcase(Dependent) = "&var" then p(&i) = ProbF ;
    if upcase(Dependent) = "&var" then af(&i) = FValue ;
   %end;
  end;

  output;
  stop;
  run;
%end;
 
%if &lsmean = YES %then %do ;
 data _lsmeans_;
  set &_lsmeans_;
 data _null_;
  set _lsmeans_;
  call symput('nclass',&lsclass);
 run;

 data &lsdata ;
 %let upp = %eval(&nclass*&ndep);
  keep  lsm1-lsm&upp lsmse1-lsmse&upp;
  array lsm(*) lsm1-lsm&upp;
  array lsmse(*) lsmse1-lsmse&upp;
  do i = 1 to totobs;
   set _lsmeans_ point=i nobs=totobs;
   %do i = 1 %to &ndep ;
    %let var = %scan(&dlist,&i);
    if upcase(Dependent) = "&var" then 
     lsm( (&i-1)*&nclass + input(&lsclass,2.0) ) = LSMean ;
    if upcase(Dependent) = "&var" then 
     lsmse( (&i-1)*&nclass + input(&lsclass,2.0) ) = StdErr ;
   %end;
  end;

  output;
  stop;
 run;
%end;

 %if %eval(&nest) gt 0 %then %do;
 data _estimates_;
  length Dependent $32;
  set &_estimates_;
 data &estdata ;
 %let upp = %eval(&nest*&ndep);
  keep e1-e&upp s1-s&upp t1-t&upp p1-p&upp;
  array e(*) e1-e&upp;
  array s(*) s1-s&upp;
  array t(*) t1-t&upp;
  array p(*) p1-p&upp;
  j=0;
  do i = 1 to totobs;
   set _estimates_ point=i nobs=totobs;
   j = j + 1;
   %do i = 1 %to &ndep ;
    %let var = %scan(&dlist,&i);
    if upcase(Dependent) = "&var" then do; 
     e((&i-1)*&nest + j) = Estimate ;
     s((&i-1)*&nest + j) = StdErr ;
     t((&i-1)*&nest + j) = tValue ;
     p((&i-1)*&nest + j) = Probt ;
    if j = &nest then j = 0;
    end;
   %end;
  end;
  output;
  stop;
  run;
%end;

 %if %eval(&ncon) gt 0 %then %do;
 data _contrasts_;
  set &_contrasts_;
 data &condata ;
 %let upp = %eval(&ncon*&ndep);
  keep df1-df&upp f1-f&upp p1-p&upp;
  array adf(*) df1-df&upp;
  array f(*) f1-f&upp;
  array p(*) p1-p&upp;
  j=0;
  do i = 1 to totobs;
   set _contrasts_ point=i nobs=totobs;
   j = j + 1;
   %do i = 1 %to &ndep ;
    %let var = %scan(&dlist,&i);
    if upcase(Dependent) = "&var" then do; 
     adf((&i-1)*&ncon + j) = DF ;
     f((&i-1)*&ncon + j) = fValue ;
     p((&i-1)*&ncon + j) = ProbF ;
    if j = &ncon then j = 0;
    end;
   %end;
  end;
  output;
  stop;
  run;
%end;

proc delete data=&_anova_ &_lsmeans_ &_estimates_ &_contrasts_ ;
run;
ods output close;
%mend ;



