
/* Used in comjuction with macro wreport
   Updated Nov 1, 2007 to allow font color 
   option color= used in move and nmove
   
   Dec 3, 2008 - replaced margin-left with text-indent. This
   is transparent to user. Both browser and Word will indent.

   May 19, 2009 - added bordercolor option

   Oct 21, 2009 - added pagebreak option (Y/N) used with mod

   Jan 6, 2009 - if center = 3 then text is right justified
   
   */



%macro makehtml (border=2, frame=box, rules=groups, cellspacing=0, 
                 bgcolor=#ffffff, textcolor=000000,bordercolor=000000,
		 fontfamily=Times New Roman,
		 fontsize=10pt,
		 fontweight=normal,
		 fontstyle=normal,
		 lineheight=125%,
		 pagebreak = Y,
                 width=100%, cellpadding=1,file=&filehtml);


/*      Parameters are based on usual html parameters:
        Table Parameters:
        border = border size around table
	frame  = type of frame around table
	rules  = where line separators are placed
	cellspacing = spacing between cells in pixels
	cellpadding = cell padding (white space in cell) in pixels
	bgcolor = background color
	textcolor = text color
	bordercolor = border color (gridlines)
	
	Text Display Parameters (These are overrided by values in move statements) 
	fontfamily = any usual font name (eg Arial)
	fonsize = points size followed by units (usually pt)
	fontweight = normal, or bold
	fontsytle = normal or italic
	lineheight = size of cell in proportion to text size
	
*/

*%put file = &file;
* If macro variable has a blank then put quotes around it;

%if %index(&fontfamily,%str( )) > 0 %then %let fontfamily = "&fontfamily";
%if %index(&fontsize,%str( ))   > 0 %then %let fontsize =   "&fontsize";
%if %index(&fontweight,%str())  > 0 %then %let fontweight = "&fontweight";
%if %index(&fontstyle,%str( ))  > 0 %then %let fontstyle =  "&fontstyle";
%if %index(&lineheight,%str( )) > 0 %then %let lineheight = "&lineheight";

%let fontfamily = %superq(fontfamily);
%let fontsize = %superq(fontsize);
%let fontweight = %superq(fontweight);
%let fontstyle = %superq(fontstyle);
%let lineheight = %superq(lineheight);

data _null_ ;
 file &file noprint lrecl=500;
 %if %index(&file,mod) > 0 and %upcase(&pagebreak) = Y %then put "<br style = 'page-break-after:always'>";;
 %if %index(&file,mod) > 0 and %upcase(&pagebreak) = N %then put "<br>";;
 put "<body bgcolor= &bgcolor text=&textcolor  >" ;
 put "<table border=&border frame=&frame rules=&rules width=&width cellspacing=&cellspacing "   
             "cellpadding=&cellpadding bordercolor=&bordercolor " 
	     "style='font-family:&fontfamily;font-size:&fontsize;font-weight:&fontweight;font-style:&fontstyle;line-height:&lineheight' >" ;
run; 
data _null_ ;
* Read in column groupings for table and append to html file;
 infile '_colgroup_.txt' pad end=eof ;
 input record $80.;
 file &file noprint mod lrecl=500;
 put record $80.;
 if eof then put '<br>';
data _temp_ ;
* Read in table data and options;
 infile '_tabledata_.txt' lrecl=500 dsd dlm='09'x missover;
 informat text $500. lineno 2. colwidth 3. colno $8. tag $8. center $8. indent $2. ;
 informat fontfamily $24. fontsize $8. fontweight $8. fontstyle $8.;
 informat textdecoration $12. lineheight $8. background $8. color $8.;
* length textalign $8.;
 input text lineno colno colwidth tag center indent fontfamily fontsize 
       fontweight fontstyle textdecoration lineheight background color;
 if lineno = . then delete;
 colnox = input (scan(colno,1,'-'), 2.);
 colnoy = input (scan(colno,2,'-'), 2.);
* Sort by line number and column number to get data in order for html table;
* Actual line numbers are ignored, tables created in sequence of order of lines;
proc sort; by lineno colnox;
data _null_;
* Append html table statements to html file;
 file &file noprint mod lrecl=500;
 retain colused 1 oldline 0 ;
 set _temp_ end=last; by lineno;
 if colnoy = 0 then colnoy = &maxcol;

* Table body tag used to group rows, skipped line numbers used to infer new section;
 if first.lineno then do;
  if lineno - oldline > 1 then put '<TBODY>';
  put "<TR>";
  colused = 1;
  oldline = lineno;
 end;

* Add non-breaking space to columns with no data;
 if colnox > colused then do ;
  do i = 1 to colnox - colused;
   put '<TD>&nbsp;</TD>' ;
  end;
 end;

 length textalign $8.;
 if colnoy ne . then span = colnoy - colnox + 1; else span =1; 
 * Added next line to create a default;
 align = "center";
 if center in('1','Y') then align = "center";
 if center in('2','N') then align = "left";
 if center in('2','N') then textalign= 'left'; else
 if center = '3' then textalign='right'; else textalign = 'center'; 
 if indent > 0 then marginleft = indent||'px'; else marginleft = 'd';

 length style $230.;
 if scan(fontfamily,2,' ') ne ' ' then fontfamily = quote(trim(fontfamily));
 style = ' ';
 if textalign  ne 'd' then style = trim(style) || 'text-align:'  || trim(textalign)  || ';'; 
 if marginleft ne 'd' then style = trim(style) || 'text-indent:' || trim(marginleft) || ';';
 if fontfamily ne 'd' then style = trim(style) || 'font-family:' || trim(fontfamily) || ';';
 if fontsize   ne 'd' then style = trim(style) || 'font-size:'   || trim(fontsize)   || ';';
 if fontstyle  ne 'd' then style = trim(style) || 'font-style:'  || trim(fontstyle)  || ';';
 if fontweight ne 'd' then style = trim(style) || 'font-weight:' || trim(fontweight) || ';';
 if background ne 'd' then style = trim(style) || 'background:'  || trim(background) || ';';
 if color ne 'd' then style = trim(style) || 'color:'  || trim(color) || ';';

 if textdecoration ne 'd' then style = trim(style) || 'text-decoration:' || trim(textdecoration) || ';';
 if lineheight ne 'd' then style = trim(style) || 'line-height:' || trim(lineheight) || ';';

 style = "'"||trim(style)||"'";

 pixwidth = &pixwidth ;
 colwidth = (10*colwidth/pixwidth)*100;
 colwidth =  int(colwidth);
  
 * Put out table data, style contains all options, changed to put them before paragraph;
 *if span = 1 then 
  put "<" tag $2. +1 "width=" colwidth +(-1) '%' +1  "align=" align +(-1)  "><P style=" style +(-1)  ">" text +(-1) "</P></" tag $2.">" ; 
 if span = 1 then 
  put "<" tag $2. +1 "width=" colwidth +(-1) '%' +1  "align=" align +(-1) " style="  style +(-1)  "><P>" text +(-1) "</P></" tag $2.">" ; 
 *if span > 1 then 
  put "<" tag $2. +1 "colspan=" span 2.  +1  "align=" align +(-1)  "><P style=" style +(-1)  '>' text +(-1) "</P></" tag $2. ">" ;
 if span > 1 then 
  put "<" tag $2. +1 "colspan=" span 2.  +1  "align=" align +(-1) " style="  style +(-1)  "><P>" text +(-1) "</P></" tag $2. ">" ;

 colused = max(colnoy,colnox) +1 ;
* Add non-breaking space to columns with no data for any remaining columns;
 if last.lineno then do;
  if colused <= &maxcol then do;
   do i = 1 to &maxcol - colused +1;
    put '<TD>&nbsp;</TD>' ;
   end;
  end;
* Add a blank line between rows of html table in html file - has no effect on viewing;
  put;
 end;

* Add end tag for table;
 if last then do;
  put "</TABLE>" ;
 end;
 run;
 * Erase work text files;
x rm '_colgroup_.txt';
x rm '_tabledata_.txt';
%mend makehtml;
