/**************************************************************************
   Updated Jan 09 - make some macro variables local
   Updated March 2001 to add ability to output statistics from estimate and 
   contrast statements.  Also, added is a repeated parameter to allow for
   the repeated statement.  The random parameter to the macro was modified
   so that the user specifies the entire random statement (see example).
   Thus, the user must modify the call to this macro for it to work.
   The parameters sub= and type= have been eliminated.
   Macro now uses output delivery system.  Works in Version 8 only.
**************************************************************************/
%macro mixedp(data=_last_,
              class=,
              dlist=, ilist=, blist=,                          
	      random=, 
	      repeated=,
	      factor=,out=&factor,
	      estimate=, estdata=,
	      contrast=, condata=,
	      lsmeans=,
	      where=,
              option=noclprint);
                                                                                
/**********************************************************************
Example:                                                                  

%mixedp(data=temp,
        class= ptid active stratum,
        dlist= sbp dbp,
	ilist= onmeds stratum active stratum*active , 
	blist= sbpbl dbpbl,                          
	lsmeans = active stratum active*stratum,
        random=int/type=un sub=ptid,
	factor= active*stratum ,
	out=interaction ,
	estimate = 'Stratum' stratum 1 -1/'Active' Active 1 -1,
	contrast = 'Stratum' stratum 1 -1/'Active' Active 1 -1,
	estdata=est, condata = con);

     MIXEDP runs PROC MIXED and stores statistics from
     the model into one or more SAS datsets. These can then be moved  
     to a report page. Default random statements and covariance                 
     structure perform standard analyses (compound symmetry or 
     equivalently random intercept only)


     Parameters to macro :
                                                                                
     data   = data set to be read                                               
     class  = class variables for MIXED                                         
     dlist  = dependent variable list - a seperate MIXED is run for             
              each variable in dlist                                            
     ilist  = independent variable list (same for each dep. var)                
              for the MIXED analyses.                                           
     blist  = (optional) if given, the i'th variable is added as                
              a covariate to the i'th analysis. Used mostly to                  
              adjust for the baseline value of the dependent                    
              variable.                                                         
     factor = independent variable(s) for which statistics are to be            
              stored. This is stored for each analysis performed.               
	      
     out    = output SAS dataset(s) in which the statistics are stored.         
              There is one dataset for each factor with one                     
              observation in each dataset.                                      
     If the factor is a class variables the output dataset contains

     p1-p(n) pvalues for that factor across dependent variables
     f1-f(n) f-statistics for that factor

     If the factor is NOT a class variables the output datset contains

     e1-e(n)   regression coefficients
     se1-se(n) standard errors
     t1-t(n)   t-statistics
     p1-p(n)   p-values

     n=the number of analyses run (= number of dep var's).             

     random = the complete random statement for proc mixed. If left
              blank no random statement will be generated.

	      Ex: random = intercept/sub=ptid g

     repeated = the complete repeated statement for proc mixed. If left
              blank no repeated statement will be generated.

	      Ex: repeated = visit/sub=ptid 

     estimate = The estimate statement if desired.  Multiple estimate 
                statments are separated by a /
     
     Ex: estimate = 'Stratum' stratum 1 -1/'Active' Active 1 -1,

     estdata = the SAS datset containing the estimate statistics. The
               variables on this dataset are:
	       
     e1-e(n)   estimate of the contrast
     se1-se(n) standard errors of the contrast
     t1-t(n)   t-statistic of the contrast
     p1-p(n)   p-value for the contrast

     n is the total number of estimate statements times the number of
     dependent variables. They are stored in the order generated. If 
     there are 2 dependent variables are 2 contrasts then  n=4 and
     
     e1 = estimate for analyses 1, contrast 1
     e2 = estimate for analyses 1, contrast 2
     e3 = estimate for analyses 2, contrast 1
     e4 = estimate for analyses 2, contrast 2

	
     contrast = The contrast statement if desired. These can 1 or >1 df
                contrasts.  Multiple contrasts are separated by a /.
		If the contrast is 1df then the syntax will be identical
		to the estimate statement
     
     Ex:contrast = 'Stratum' stratum 1 -1/'Active' Active 1 -1,

	>1 df contrast are given like this:
	contrast = %str('2 df test' Active 1 0 -1, Active 1 -1 0);
	      
     condata = the SAS datset containing the contrast statistics. The
               variables on this dataset are:
	       
     f1-f(n)   F-stat for the contrast
     p1-p(n)   p-value for the contrast
     df1-df(n) df for the contrast

     n is the total number of contrast statments times the number of
     dependent variables. They are stored in the same order as in the
     estimate statements (see above).

     lsmeans = if desired include the class factors desired (see example)
               No dataset is created from this statement.
     where   = where clause to subset observations                              
               (set where to "where sex = 1" to do analyses on men)
     option  = options added to the PROC MIXED statement.

                                                                                
**********************************************************************/

ods output SolutionF (match_all=_sol_ persist=proc) = _sol_;
ods output Tests3    (match_all=_tests_ persist=proc) = _tests_ ;
ods output LSMeans   (match_all=_lsmeans_ persist=proc)= _lsmeans_ ;
ods output Estimates (match_all=_estimates_ persist=proc)= _estimates_ ;
ods output Contrasts (match_all=_contrasts_ persist=proc)= _contrasts_ ;

data _null_;                                                                    

%local _lsmeans_ _estimates_ _contrasts_ i j k dep bas ;
%let dlist = %upcase(&dlist) ;                                                  
%let factor = %upcase(&factor) ;                                                
%ntok(&factor,nfactor,vname=factor);
%ntok(&out   ,nout   ,vname=out);                                               
%ntok(&dlist ,ndep);                                                            
%ntok(&blist ,nbas);
%ntok(&estimate ,nest,vname=estimate,token=%str(/));
%ntok(&contrast ,ncon,vname=contrast,token=%str(/));
                                               
%if &class eq %str() or &dlist eq %str() or %length(&ilist) = 0                 
    or %length(&factor)  eq 0  or &out   eq %str() %then %do ;                  
 ERROR '*** SOME PARAMETERS NOT SPECIFIED - SEE MACRO CALL ***';                
 ABORT;                                                                         
 ENDSAS;                                                                        
%end;                                                                           
                                                                                
%if &blist ne %str() %then %do;                                                 
%if &ndep ne &nbas %then %do;                                                   
 ERROR '*** NUMBER OF DEPENDENT AND BASELINE VARIABLES NOT = ***';              
 ABORT;                                                                         
 ENDSAS;                                                                        
%end;                                                                           
%end;                                                                           
                                                                                
%if &nout   ne &nfactor %then %do ;                                             
 ERROR '*** NUMBERS OF FACTORS AND OUTPUT DATASETS NOT =  ***';                 
 ABORT;                                                                         
 ENDSAS;                                                                        
%end;                                                                           
                                                                                
                                                                                
 %do i = 1 %to &ndep;                                                           
  %let dep = %scan(&dlist,&i);                                                  
  %let bas = %scan(&blist,&i);                                                  
  proc mixed data=&data &option ;
   class &class;                                                                
   model &dep = &ilist &bas  /s  ;                                              

   %if %length(&random) > 0 %then %do;
   random &random ;
   %end;

   %if %length(&repeated) > 0 %then %do;
   repeated &repeated ;
   %end;
   
   %if %length(&lsmeans) > 0 %then %do;
    lsmeans &lsmeans;
   %end;
   
   %if %eval(&nest) gt 0 %then %do;
   %do j = 1 %to &nest; 
    estimate &&estimate&j ;
   %end;
   %end;

   %if %eval(&ncon) gt 0 %then %do;
   %do j = 1 %to &ncon;
    contrast &&contrast&j; 
    %end;
   %end;
   
   %if %length(&where) ne 0 %then %do;
    &where;
   %end;

 %end;
  run;
 data _sol_ ;
  set &_sol_;
 data _tests_ ;
  set &_tests_;
 run;

 %if %eval(&nest) gt 0 %then %do;
 data _estimates_;
  set &_estimates_;
 data &estdata ;
 %let upp = %eval(&nest*&ndep);
  keep e1-e&upp se1-se&upp t1-t&upp p1-p&upp ;
  array e(*) e1-e&upp;
  array se(*) se1-se&upp;
  array t(*) t1-t&upp;
  array p(*) p1-p&upp;
  do i = 1 to totobs;
   set _estimates_ point=i nobs=totobs;
    e(i) = Estimate ;
    se(i) = StdErr ;
    t(i) = tValue ;
    p(i) = Probt ;
  end;
  output;
  stop;
  run;
 %end;

 %if %eval(&ncon) gt 0 %then %do;
 data _contrasts_;
  set &_contrasts_;
 data &condata ;
 %let upp = %eval(&ncon*&ndep);
  keep df1-df&upp f1-f&upp p1-p&upp;
  array adf(*) df1-df&upp;
  array f(*) f1-f&upp;
  array p(*) p1-p&upp;
  do i = 1 to totobs;
   set _contrasts_ point=i nobs=totobs;
    adf(i) = NumDF ;
    f(i) = FValue ;
    p(i) = ProbF ;
  end;
  output;
  stop;
  run;
 %end;

%do j = 1 %to &nfactor ;                                                        
 data &&out&j;                                                                  
  %if %index(%upcase(&class), &&factor&j) gt 0 or
      %index(&&factor&j,*) gt 0 %then %do;
  set _tests_ ;
  if upcase(effect) = "&&factor&j" ;
 data &&out&j;
  keep effect p1-p&ndep f1-f&ndep ;
  array p(*) p1-p&ndep;                                                         
  array fa(*) f1-f&ndep;
  do i = 1 to &ndep ;                                                           
   set &&out&j point=i ;                                                        
     p(i) = ProbF ;
     fa(i) = FValue ;
  end;                                                                          
  output;                                                                       
  stop;                                                                         
  %end;
  %else %do;
  set _sol_;
  if upcase(effect) = "&&factor&j" ;
 data &&out&j;                                                                  
  keep effect p1-p&ndep e1-e&ndep p1-p&ndep se1-se&ndep t1-t&ndep;
  array e(*) e1-e&ndep;                                                         
  array p(*) p1-p&ndep;                                                         
  array se(*) se1-se&ndep;  
  array tv(*) t1-t&ndep;                                                        
  do i = 1 to &ndep ;                                                           
   set &&out&j point=i ;                                                        
     e(i) = Estimate ;
     p(i) = Probt ;
     se(i) = StdErr;
     tv(i) = tValue;
  end;                                                                          
  output;                                                                       
  stop;                                                                         
  %end;
  run;                                                                          
%end;                                                                           
proc delete data= _sol_ _tests_ _lsmeans_ _estimates_ _contrasts_ ;
ods output close;
%mend ;                                                                         
